/*
 * Copyright 2010-2013, CloudBees Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.cloudbees.sdk.commands.db;

import com.cloudbees.api.*;
import com.cloudbees.sdk.cli.BeesClientFactory;
import com.cloudbees.sdk.commands.Command;
import com.cloudbees.sdk.utils.Helper;

import javax.inject.Inject;
import java.io.IOException;
import java.util.Map;

/**
 * @author Fabian Donze
 */
public abstract class DatabaseBase extends Command {
    @Inject
    BeesClientFactory beesClientFactory;

    private String databaseName;

    private String account;

    public DatabaseBase() {
        setArgumentExpected(1);
    }

    public void setDatabaseName(String databaseName) {
        String[] parts = databaseName.split("/");
        if (parts.length > 1) setAccount(parts[0]);
        this.databaseName = databaseName;
    }

    protected String getDatabaseName() throws IOException {
        if (databaseName == null) databaseName = Helper.promptFor("Database name: ", true);

        String[] appIdParts = databaseName.split("/");
        if (appIdParts.length < 2) {
            String defaultAppDomain = getAccount();
            if (defaultAppDomain != null && !defaultAppDomain.equals("")) {
                databaseName = defaultAppDomain + "/" + databaseName;
 //           } else {
 //               throw new RuntimeException("default database account could not be determined");
            }
        }

        return databaseName;
    }

    protected String getAccount() throws IOException {
        if (account == null) account = beesClientFactory.getConfigProperties().getProperty("bees.project.app.domain");
        return account;
    }

    public void setAccount(String account) {
        this.account = account;
    }

    @Override
    protected String getUsageMessage() {
        return "DATABASE_NAME";
    }

    @Override
    protected boolean preParseCommandLine() {
        return true;
    }

    @Override
    protected boolean postParseCheck() {
        if (super.postParseCheck()) {
            setDatabaseName(getParameters().get(0));
            return true;
        }
        return false;
    }

    protected AccountRegionInfo getDatabaseRegionInfo(BeesClient client, String dbId) {
        try {
            ServiceResourceInfo resourceInfo = client.serviceResourceInfo("cb-db", dbId);
            Map<String, String> config= resourceInfo.getConfig();
            if (config != null) {
                String region = config.get("region");
                if (region != null) {
                    String parts[] = dbId.split("/");
                    AccountRegionListResponse res = client.accountRegionList(parts[0], null);
                    return getRegionInfo(res, region);
                }
            }
        } catch (Exception e) {
            if (!e.getMessage().toLowerCase().contains("no such resource"))
                System.err.println("Error: " + e.getMessage());
        }

        return null;
    }

    /**
     * Returns a BeesClient for the API end point, the database was created with
     * @param dbID  The Database ID
     * @return BeesClient
     * @throws IOException
     */
    protected <T extends BeesClient> T getBeesClient(Class<T> clientType, String dbID) throws IOException {
        T client = beesClientFactory.get(clientType);
        AccountRegionInfo endPoint = getDatabaseRegionInfo(client, dbID);
        if (endPoint != null) {
            String apiUrl = endPoint.getSettings().get("api.url");
            BeesClientConfiguration clientConfiguration = client.getBeesClientConfiguration();
            String currentApiUrl = clientConfiguration.getServerApiUrl();
            if (!currentApiUrl.equalsIgnoreCase(apiUrl)) {
                System.err.println(String.format("WARNING: Database [%s] defined in the [%s] region, using [%s] API end point",
                        dbID, endPoint.getName(), endPoint.getName()));
                beesClientFactory.server = apiUrl;
                client = beesClientFactory.get(clientType);
            }
        }
        return client;
    }

    private AccountRegionInfo getRegionInfo(AccountRegionListResponse regions, String region) {
        for (AccountRegionInfo regionInfo : regions.getRegions()) {
            if (regionInfo.getName().equalsIgnoreCase(region)) {
                return regionInfo;
            }
        }
        return null;
    }

}
